@@ -1,10 +1,12 @@
+import types
 """
 Classes and stuff that handle querying the interface for a passed in Orm class
 """
-import types
-import copy
-
-from .utils import make_list
+#    query.table("table_name").is_foo(1).desc_bar().limit(10).page(2).get()
+#    query.table("table_name").is_foo(1).desc_bar().set_limit(10).set_page(2).get()
+#    query.table("table_name").is_foo(1).desc_bar().with_limit(10).with_page(2).get()
+#    query.table("table_name").is_foo(1).desc_bar().use_limit(10).use_page(2).get()
+#    query.table("table_name").is_foo(1).desc_bar().limit_to(10).on_page(2).with_offset(5).get()
 
 
 class Iterator(object):
@@ -36,8 +38,7 @@ class Iterator(object):
         self.results = results
         self.orm = orm
         self.has_more = has_more
-        self.query = copy.deepcopy(query)
-        self._values = False
+        self.query = query
         self.reset()
 
     def reset(self):
@@ -51,72 +52,48 @@ class Iterator(object):
         similar to the dict.values() method, this will only return the selected fields
         in a tuple
 
-        return -- self -- each iteration will return just the field values in
+        return -- generator -- each iteration will return just the field values in
             the order they were selected, if you only selected one field, than just that field
             will be returned, if you selected multiple fields than a tuple of the fields in
             the order you selected them will be returned
         """
-        self._values = True
-        self.field_names = self.query.fields_select
-        self.fcount = len(self.field_names)
-        if not self.fcount:
-            raise ValueError("no select fields were set, so cannot iterate values")
+        field_names = self.query.fields_select
+        fcount = len(field_names)
+        if fcount:
+            for x in self.results:
+                field_vals = [x.get(fn, None) for fn in field_names]
+                yield field_vals if fcount > 1 else field_vals[0]
 
-        return self
+        else:
+            raise ValueError("no select fields were set, so cannot iterate values")
 
     def __iter__(self):
         self.reset()
         return self
 
-    def __nonzero__(self):
-        return True if self.count() else False
-
     def __len__(self):
-        return self.count()
-
-    def count(self):
-        """list interface compatibility"""
         return len(self.results)
 
     def __getitem__(self, k):
         k = int(k)
         return self._get_result(self.results[k])
 
-    def pop(self, k=-1):
-        """list interface compatibility"""
-        k = int(k)
-        return self._get_result(self.results.pop(k))
-
-    def reverse(self):
-        """list interface compatibility"""
-        self.results.reverse()
-        self.reset()
-
-    def sort(self, *args, **kwargs):
-        """list interface compatibility"""
-        self.results.sort(*args, **kwargs)
-        self.reset()
-
     def __getattr__(self, k):
         """
         this allows you to focus in on certain fields of results
 
         It's just an easier way of doing: (getattr(x, k, None) for x in self)
         """
+        pout.v(k)
         field_name = self.orm.schema.field_name(k)
         return (getattr(r, field_name, None) for r in self)
 
     def _get_result(self, d):
         r = None
-        if self._values:
-            field_vals = [d.get(fn, None) for fn in self.field_names]
-            r = field_vals if self.fcount > 1 else field_vals[0]
-
+        if self.orm:
+            r = self.orm.populate(d)
         else:
-            if self.orm:
-                r = self.orm.populate(d)
-            else:
-                r = d
+            r = d
 
         return r
 
@@ -137,73 +114,16 @@ class AllIterator(Iterator):
         self.offset = offset
         super(AllIterator, self).__init__(results=[], orm=query.orm, query=query)
 
-    def __getitem__(self, k):
-        v = None
-        k = int(k)
-        lower_bound = self.offset
-        upper_bound = lower_bound + self.chunk_limit
-        if k >= lower_bound and k < upper_bound:
-            # k should be in this result set
-            i = k - lower_bound
-            v = self.results[i]
-
-        else:
-            # k is not in here, so let's just grab it
-            q = copy.deepcopy(self.query)
-            orm = q.set_offset(k).get_one()
-            if orm:
-                v = self._get_result(orm.fields)
-            else:
-                raise IndexError("results index out of range")
-
-        return v
-
-    def __len__(self):
-        ret = 0
-        if self.results.has_more:
-            # we need to do a count query
-            q = copy.deepcopy(self.query)
-            q.set_limit(0).set_offset(0)
-            ret = q.count()
-        else:
-            ret = (self.offset - self.start_offset) + len(self.results)
-
-        return ret
-
     def __iter__(self):
         has_more = True
-        self.reset()
         while has_more:
+            self.results = self.query.set_offset(self.offset).get(self.chunk_limit)
             has_more = self.results.has_more
             for r in self.results:
                 yield r
 
             self.offset += self.chunk_limit
-            self._set_results()
 
-    def _set_results(self):
-        self.results = self.query.set_offset(self.offset).get(self.chunk_limit)
-        if self._values:
-            self.results = self.results.values()
-
-    def reset(self):
-        set_results = False
-        if hasattr(self, 'start_offset'):
-            set_results = self.offset != self.start_offset
-        else:
-            self.start_offset = self.offset
-            set_results = True
-
-        if set_results:
-            self.offset = self.start_offset
-            self._set_results()
-
-        else:
-            self.results.reset()
-
-    def values(self):
-        self.results = self.results.values()
-        return super(AllIterator, self).values()
 
 class Query(object):
     """
@@ -234,50 +154,16 @@ class Query(object):
         self.bounds = {}
         self.args = args
         self.kwargs = kwargs
-        self.can_get = True
 
     def __iter__(self):
         return self.get()
 
-    def copy(self):
-        """nice handy wrapper around the deepcopy"""
-        return copy.deepcopy(self)
-
-    def __deepcopy__(self, memodict={}):
-        q = type(self)(self.orm)
-        for key, val in self.__dict__.iteritems():
-            if isinstance(val, types.ListType):
-                setattr(q, key, list(val))
-
-            elif isinstance(val, types.DictType):
-                setattr(q, key, dict(val))
-
-            elif isinstance(val, types.TupleType):
-                setattr(q, key, tuple(val))
-
-            else:
-                setattr(q, key, val)
-
-        return q
-
-    def select_field(self, field_name):
-        """set a field to be selected"""
-        return self.set_field(field_name, None)
-
-    def select_fields(self, *fields):
-        """set multiple fields to be selected"""
-        if fields:
-            if not isinstance(fields[0], types.StringTypes): 
-                fields = list(fields[0]) + list(fields)[1:]
-
-        for field_name in fields:
-            self.set_field(field_name)
-
     def set_field(self, field_name, field_val=None):
         """
         set a field into .fields attribute
 
-        n insert/update queries, these are the fields that will be inserted/updated into the db
+        this has a dual role, in select queries, these are the select fields, but in insert/update
+        queries, these are the fields that will be inserted/updated into the db
         """
         self.fields_set.append([field_name, field_val])
         return self
@@ -310,14 +196,12 @@ class Query(object):
 
         return self
 
-    def is_field(self, field_name, *field_val, **field_kwargs):
-        fv = field_val[0] if field_val else None
-        self.fields_where.append(["is", field_name, fv, field_kwargs])
+    def is_field(self, field_name, field_val):
+        self.fields_where.append(["is", field_name, field_val])
         return self
 
-    def not_field(self, field_name, *field_val, **field_kwargs):
-        fv = field_val[0] if field_val else None
-        self.fields_where.append(["not", field_name, fv, field_kwargs])
+    def not_field(self, field_name, field_val):
+        self.fields_where.append(["not", field_name, field_val])
         return self
 
     def between_field(self, field_name, low, high):
@@ -325,70 +209,39 @@ class Query(object):
         self.gte_field(field_name, high)
         return self
 
-    def lte_field(self, field_name, *field_val, **field_kwargs):
-        fv = field_val[0] if field_val else None
-        self.fields_where.append(["lte", field_name, fv, field_kwargs])
+    def lte_field(self, field_name, field_val):
+        self.fields_where.append(["lte", field_name, field_val])
         return self
 
-    def lt_field(self, field_name, *field_val, **field_kwargs):
-        fv = field_val[0] if field_val else None
-        self.fields_where.append(["lt", field_name, fv, field_kwargs])
+    def lt_field(self, field_name, field_val):
+        self.fields_where.append(["lt", field_name, field_val])
         return self
 
-    def gte_field(self, field_name, *field_val, **field_kwargs):
-        fv = field_val[0] if field_val else None
-        self.fields_where.append(["gte", field_name, fv, field_kwargs])
+    def gte_field(self, field_name, field_val):
+        self.fields_where.append(["gte", field_name, field_val])
         return self
 
-    def gt_field(self, field_name, *field_val, **field_kwargs):
-        fv = field_val[0] if field_val else None
-        self.fields_where.append(["gt", field_name, fv, field_kwargs])
+    def gt_field(self, field_name, field_val):
+        self.fields_where.append(["gt", field_name, field_val])
         return self
 
-    def in_field(self, field_name, *field_vals, **field_kwargs):
+    def in_field(self, field_name, field_vals):
         """
         field_vals -- list -- a list of field_val values
         """
-        fv = make_list(field_vals[0]) if field_vals else None
-        if field_kwargs:
-            for k in field_kwargs:
-                if not field_kwargs[k]:
-                    raise ValueError("Cannot IN an empty list")
-
-                field_kwargs[k] = make_list(field_kwargs[k])
-
-        else:
-            if not fv: self.can_get = False
-
-        self.fields_where.append(["in", field_name, fv, field_kwargs])
+        assert field_vals, "Cannot IN an empty list"
+        self.fields_where.append(["in", field_name, list(field_vals)])
         return self
 
-    def nin_field(self, field_name, *field_vals, **field_kwargs):
+    def nin_field(self, field_name, field_vals):
         """
         field_vals -- list -- a list of field_val values
         """
-        fv = make_list(field_vals[0]) if field_vals else None
-        if field_kwargs:
-            for k in field_kwargs:
-                if not field_kwargs[k]:
-                    raise ValueError("Cannot IN an empty list")
-
-                field_kwargs[k] = make_list(field_kwargs[k])
-
-        else:
-            if not fv: self.can_get = False
-
-        self.fields_where.append(["nin", field_name, fv, field_kwargs])
+        assert field_vals, "Cannot NIN an empty list"
+        self.fields_where.append(["nin", field_name, list(field_vals)])
         return self
 
-    def sort_field(self, field_name, direction, field_vals=None):
-        """
-        sort this query by field_name in directrion
-
-        field_name -- string -- the field to sort on
-        direction -- integer -- negative for DESC, positive for ASC
-        field_vals -- list -- the order the rows should be returned in
-        """
+    def sort_field(self, field_name, direction):
         if direction > 0:
             direction = 1
         elif direction < 0:
@@ -396,15 +249,15 @@ class Query(object):
         else:
             raise ValueError("direction {} is undefined".format(direction))
 
-        self.fields_sort.append([direction, field_name, list(field_vals) if field_vals else field_vals])
+        self.fields_sort.append([direction, field_name])
         return self
 
-    def asc_field(self, field_name, field_vals=None):
-        self.sort_field(field_name, 1, field_vals)
+    def asc_field(self, field_name):
+        self.sort_field(field_name, 1)
         return self
 
-    def desc_field(self, field_name, field_vals=None):
-        self.sort_field(field_name, -1, field_vals)
+    def desc_field(self, field_name):
+        self.sort_field(field_name, -1)
         return self
 
     def __getattr__(self, method_name):
@@ -426,8 +279,6 @@ class Query(object):
 
     def _split_method(self, method_name):
         command, field_name = method_name.split(u"_", 1)
-        if self.orm: field_name = self.orm.schema.field_name(field_name)
-
         return command, field_name
 
     def set_limit(self, limit):
@@ -445,6 +296,7 @@ class Query(object):
         return self
 
     def get_bounds(self):
+
         limit = offset = page = limit_paginate = 0
         if "limit" in self.bounds and self.bounds["limit"] > 0:
             limit = self.bounds["limit"]
@@ -465,10 +317,6 @@ class Query(object):
     def has_bounds(self):
         return len(self.bounds) > 0
 
-    def has_limit(self):
-        limit = self.bounds.get('limit', 0)
-        return limit > 0
-
     def get(self, limit=None, page=None):
         """
         get results from the db
@@ -485,7 +333,6 @@ class Query(object):
         if limit_paginate:
             self.set_limit(limit_paginate)
 
-        self.default_val = []
         results = self._query('get')
 
         if limit_paginate:
@@ -510,80 +357,20 @@ class Query(object):
 
     def get_one(self):
         """get one row from the db"""
-        self.default_val = None
-        o = self.default_val
+        o = None
         d = self._query('get_one')
         if d:
             o = self.orm.populate(d)
         return o
 
-    def values(self, limit=None, page=None):
-        """
-        convenience method to get just the values from the query (same as get().values())
-
-        if you want to get all values, you can use: self.all().values()
-        """
-        return self.get(limit=limit, page=page).values()
-
-    def value(self):
-        """convenience method to just get one value or tuple of values for the query"""
-        field_vals = None
-        field_names = self.fields_select
-        fcount = len(field_names)
-        if fcount:
-            d = self._query('get_one')
-            if d:
-                field_vals = [d.get(fn, None) for fn in field_names]
-                if fcount == 1:
-                    field_vals = field_vals[0]
-
-        else:
-            raise ValueError("no select fields were set, so cannot return value")
-
-        return field_vals
-
-    def pks(self, limit=None, page=None):
-        """convenience method for setting select_pk().values() since this is so common"""
-        self.fields_set = []
-        return self.select_pk().values(limit, page)
-
-    def pk(self):
-        """convenience method for setting select_pk().value() since this is so common"""
-        self.fields_set = []
-        return self.select_pk().value()
-
-    def get_pks(self, field_vals):
-        """convenience method for running in__id([...]).get() since this is so common"""
-        field_name = self.orm.schema.pk
-        return self.in_field(field_name, field_vals).get()
-
     def get_pk(self, field_val):
-        """convenience method for running is_pk(_id).get_one() since this is so common"""
+        """convenience method for running is__id(_id).get_one() since this is so common"""
         field_name = self.orm.schema.pk
         return self.is_field(field_name, field_val).get_one()
 
-    def first(self):
-        """convenience method for running asc__id().get_one()"""
-        return self.asc__id().get_one()
-
-    def last(self):
-        """convenience method for running desc__id().get_one()"""
-        return self.desc__id().get_one()
-
     def count(self):
         """return the count of the criteria"""
-
-        # count queries shouldn't care about sorting
-        fields_sort = self.fields_sort
-        self.fields_sort = []
-
-        self.default_val = 0
-        ret = self._query('count')
-
-        # restore previous values now that count is done
-        self.fields_sort = fields_sort
-
-        return ret
+        return self._query('count')
 
     def has(self):
         """returns true if there is atleast one row in the db matching the query, False otherwise"""
@@ -592,34 +379,26 @@ class Query(object):
 
     def set(self):
         """persist the .fields using .fields_where (if available)"""
-        self.default_val = None
         return self._query('set')
 
     def delete(self):
         """remove fields matching the where criteria"""
-        self.default_val = None
         return self._query('delete')
 
-    def raw(self, query_str, *query_args, **query_options):
+    def get_query(self, query_str, *query_args, **query_options):
         """
-        use the interface.query() method to pass in your own raw query without
-        any processing
-
-        NOTE -- This will allow you to make any raw query and will usually return
-        raw results, it won't wrap those results in a prom.Orm iterator instance
-        like other methods like .all() and .get()
-
-        query_str -- string -- the raw query for whatever the backend interface is
-        query_args -- list -- if you have named parameters, you can pass in the values
-        **query_options -- dict -- key:val options for the backend, these are very backend specific
-        return -- mixed -- depends on the backend and the type of query
+        use the interface query method to pass in your own raw query
         """
         i = self.orm.interface
         return i.query(query_str, *query_args, **query_options)
 
     def _query(self, method_name):
-        if not self.can_get: return self.default_val
         i = self.orm.interface
         s = self.orm.schema
         return getattr(i, method_name)(s, self)
 
+    def _create_orm(self, d):
+        o = self.orm(**d)
+        o.reset_modified()
+        return o
+
